# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import GenerateDetailedCostReportOperationStatusOperations  # type: ignore
from ._operations import Operations  # type: ignore
from ._operations import BudgetsOperations  # type: ignore
from ._operations import ExportsOperations  # type: ignore
from ._operations import GenerateDetailedCostReportOperationResultsOperations  # type: ignore
from ._operations import ViewsOperations  # type: ignore
from ._operations import AlertsOperations  # type: ignore
from ._operations import ScheduledActionsOperations  # type: ignore
from ._operations import SettingsOperations  # type: ignore
from ._operations import GenerateCostDetailsReportOperations  # type: ignore
from ._operations import CostAllocationRulesOperations  # type: ignore
from ._operations import BenefitRecommendationsOperations  # type: ignore
from ._operations import BenefitUtilizationSummariesOperations  # type: ignore
from ._operations import GenerateBenefitUtilizationSummariesReportOperations  # type: ignore
from ._operations import GenerateDetailedCostReportOperations  # type: ignore
from ._operations import ForecastOperations  # type: ignore
from ._operations import DimensionsOperations  # type: ignore
from ._operations import QueryOperations  # type: ignore
from ._operations import GenerateReservationDetailsReportOperations  # type: ignore
from ._operations import PriceSheetOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "GenerateDetailedCostReportOperationStatusOperations",
    "Operations",
    "BudgetsOperations",
    "ExportsOperations",
    "GenerateDetailedCostReportOperationResultsOperations",
    "ViewsOperations",
    "AlertsOperations",
    "ScheduledActionsOperations",
    "SettingsOperations",
    "GenerateCostDetailsReportOperations",
    "CostAllocationRulesOperations",
    "BenefitRecommendationsOperations",
    "BenefitUtilizationSummariesOperations",
    "GenerateBenefitUtilizationSummariesReportOperations",
    "GenerateDetailedCostReportOperations",
    "ForecastOperations",
    "DimensionsOperations",
    "QueryOperations",
    "GenerateReservationDetailsReportOperations",
    "PriceSheetOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cdn import CdnManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCdnManagementLogAnalyticsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CdnManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_log_analytics_metrics(self, resource_group):
        response = self.client.log_analytics.get_log_analytics_metrics(
            resource_group_name=resource_group.name,
            profile_name="str",
            metrics=["str"],
            date_time_begin="2020-02-20 00:00:00",
            date_time_end="2020-02-20 00:00:00",
            granularity="str",
            custom_domains=["str"],
            protocols=["str"],
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_log_analytics_rankings(self, resource_group):
        response = self.client.log_analytics.get_log_analytics_rankings(
            resource_group_name=resource_group.name,
            profile_name="str",
            rankings=["str"],
            metrics=["str"],
            max_ranking=0,
            date_time_begin="2020-02-20 00:00:00",
            date_time_end="2020-02-20 00:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_log_analytics_locations(self, resource_group):
        response = self.client.log_analytics.get_log_analytics_locations(
            resource_group_name=resource_group.name,
            profile_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_log_analytics_resources(self, resource_group):
        response = self.client.log_analytics.get_log_analytics_resources(
            resource_group_name=resource_group.name,
            profile_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_waf_log_analytics_metrics(self, resource_group):
        response = self.client.log_analytics.get_waf_log_analytics_metrics(
            resource_group_name=resource_group.name,
            profile_name="str",
            metrics=["str"],
            date_time_begin="2020-02-20 00:00:00",
            date_time_end="2020-02-20 00:00:00",
            granularity="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_log_analytics_get_waf_log_analytics_rankings(self, resource_group):
        response = self.client.log_analytics.get_waf_log_analytics_rankings(
            resource_group_name=resource_group.name,
            profile_name="str",
            metrics=["str"],
            date_time_begin="2020-02-20 00:00:00",
            date_time_end="2020-02-20 00:00:00",
            max_ranking=0,
            rankings=["str"],
        )

        # please add some check logic here by yourself
        # ...



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
import datetime
import email.utils
import urllib.parse
from typing import Optional, Set, cast, Union, Tuple
from urllib.parse import urlparse

from azure.core.pipeline.transport import (
    HttpResponse as LegacyHttpResponse,
    AsyncHttpResponse as LegacyAsyncHttpResponse,
    HttpRequest as LegacyHttpRequest,
)
from azure.core.rest import HttpResponse, AsyncHttpResponse, HttpRequest


from ...utils._utils import case_insensitive_dict, CaseInsensitiveSet
from .. import PipelineResponse


AllHttpResponseType = Union[HttpResponse, LegacyHttpResponse, AsyncHttpResponse, LegacyAsyncHttpResponse]
HTTPRequestType = Union[HttpRequest, LegacyHttpRequest]


def _parse_http_date(text: str) -> datetime.datetime:
    """Parse a HTTP date format into datetime.

    :param str text: Text containing a date in HTTP format
    :rtype: datetime.datetime
    :return: The parsed datetime
    """
    parsed_date = email.utils.parsedate_tz(text)
    if not parsed_date:
        raise ValueError("Invalid HTTP date")
    tz_offset = cast(int, parsed_date[9])  # Look at the code, tz_offset is always an int, at worst 0
    return datetime.datetime(*parsed_date[:6], tzinfo=datetime.timezone(datetime.timedelta(seconds=tz_offset)))


def parse_retry_after(retry_after: str) -> float:
    """Helper to parse Retry-After and get value in seconds.

    :param str retry_after: Retry-After header
    :rtype: float
    :return: Value of Retry-After in seconds.
    """
    delay: float  # Using the Mypy recommendation to use float for "int or float"
    try:
        delay = float(retry_after)
    except ValueError:
        # Not an integer? Try HTTP date
        retry_date = _parse_http_date(retry_after)
        delay = (retry_date - datetime.datetime.now(retry_date.tzinfo)).total_seconds()
    return max(0, delay)


def get_retry_after(response: PipelineResponse[HTTPRequestType, AllHttpResponseType]) -> Optional[float]:
    """Get the value of Retry-After in seconds.

    :param response: The PipelineResponse object
    :type response: ~azure.core.pipeline.PipelineResponse
    :return: Value of Retry-After in seconds.
    :rtype: float or None
    """
    headers = case_insensitive_dict(response.http_response.headers)
    retry_after = headers.get("retry-after")
    if retry_after:
        return parse_retry_after(retry_after)
    for ms_header in ["retry-after-ms", "x-ms-retry-after-ms"]:
        retry_after = headers.get(ms_header)
        if retry_after:
            parsed_retry_after = parse_retry_after(retry_after)
            return parsed_retry_after / 1000.0
    return None


def get_domain(url: str) -> str:
    """Get the domain of an url.

    :param str url: The url.
    :rtype: str
    :return: The domain of the url.
    """
    return str(urlparse(url).netloc).lower()


def get_challenge_parameter(headers, challenge_scheme: str, challenge_parameter: str) -> Optional[str]:
    """
    Parses the specified parameter from a challenge header found in the response.

    :param dict[str, str] headers: The response headers to parse.
    :param str challenge_scheme: The challenge scheme containing the challenge parameter, e.g., "Bearer".
    :param str challenge_parameter: The parameter key name to search for.
    :return: The value of the parameter name if found.
    :rtype: str or None
    """
    header_value = headers.get("WWW-Authenticate")
    if not header_value:
        return None

    scheme = challenge_scheme
    parameter = challenge_parameter
    header_span = header_value

    # Iterate through each challenge value.
    while True:
        challenge = get_next_challenge(header_span)
        if not challenge:
            break
        challenge_key, header_span = challenge
        if challenge_key.lower() != scheme.lower():
            continue
        # Enumerate each key-value parameter until we find the parameter key on the specified scheme challenge.
        while True:
            parameters = get_next_parameter(header_span)
            if not parameters:
                break
            key, value, header_span = parameters
            if key.lower() == parameter.lower():
                return value

    return None


def get_next_challenge(header_value: str) -> Optional[Tuple[str, str]]:
    """
    Iterates through the challenge schemes present in a challenge header.

    :param str header_value: The header value which will be sliced to remove the first parsed challenge key.
    :return: The parsed challenge scheme and the remaining header value.
    :rtype: tuple[str, str] or None
    """
    header_value = header_value.lstrip(" ")
    end_of_challenge_key = header_value.find(" ")

    if end_of_challenge_key < 0:
        return None

    challenge_key = header_value[:end_of_challenge_key]
    header_value = header_value[end_of_challenge_key + 1 :]

    return challenge_key, header_value


def get_next_parameter(header_value: str, separator: str = "=") -> Optional[Tuple[str, str, str]]:
    """
    Iterates through a challenge header value to extract key-value parameters.

    :param str header_value: The header value after being parsed by get_next_challenge.
    :param str separator: The challenge parameter key-value pair separator, default is '='.
    :return: The next available challenge parameter as a tuple (param_key, param_value, remaining header_value).
    :rtype: tuple[str, str, str] or None
    """
    space_or_comma = " ,"
    header_value = header_value.lstrip(space_or_comma)

    next_space = header_value.find(" ")
    next_separator = header_value.find(separator)

    if next_space < next_separator and next_space != -1:
        return None

    if next_separator < 0:
        return None

    param_key = header_value[:next_separator].strip()
    header_value = header_value[next_separator + 1 :]

    quote_index = header_value.find('"')

    if quote_index >= 0:
        header_value = header_value[quote_index + 1 :]
        param_value = header_value[: header_value.find('"')]
    else:
        trailing_delimiter_index = header_value.find(" ")
        if trailing_delimiter_index >= 0:
            param_value = header_value[:trailing_delimiter_index]
        else:
            param_value = header_value

    if header_value != param_value:
        header_value = header_value[len(param_value) + 1 :]

    return param_key, param_value, header_value


def sanitize_url(url: str, allowed_query_params: Set[str], redacted_placeholder: str = "REDACTED") -> str:
    """Redact query parameter values not in the allowlist.

    :param str url: The URL to sanitize.
    :param set[str] allowed_query_params: Set of query parameter names whose values should not be redacted.
        If a :class:`~azure.core.utils._utils.CaseInsensitiveSet` is provided, lookups are case-insensitive
        without per-call normalization.
    :param str redacted_placeholder: The placeholder to use for redacted values.
    :return: The sanitized URL with redacted query parameter values.
    :rtype: str
    """
    parsed_url = urllib.parse.urlparse(url)
    if not parsed_url.query:
        return url

    # Operate on the raw query string to preserve original percent-encoding.
    if not isinstance(allowed_query_params, CaseInsensitiveSet):
        allowed_query_params = CaseInsensitiveSet(allowed_query_params)

    parts = []
    for param in parsed_url.query.split("&"):
        eq_idx = param.find("=")
        if eq_idx == -1:
            # No value to redact, keep as-is.
            parts.append(param)
        else:
            key = param[:eq_idx]
            parts.append(param if key in allowed_query_params else f"{key}={redacted_placeholder}")

    sanitized_query = "&".join(parts)
    sanitized_url = parsed_url._replace(query=sanitized_query)
    return urllib.parse.urlunparse(sanitized_url)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, Callable, Dict, IO, Iterable, Iterator, Optional, TypeVar, Union, cast, overload
import urllib.parse

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling

from .. import models as _models
from .._serialization import Serializer

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_request(subscription_id: str, *, skip_token: Optional[str] = None, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.DataBox/jobs")
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if skip_token is not None:
        _params["$skipToken"] = _SERIALIZER.query("skip_token", skip_token, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_mark_devices_shipped_request(
    job_name: str, resource_group_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}/markDevicesShipped",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_by_resource_group_request(
    resource_group_name: str, subscription_id: str, *, skip_token: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if skip_token is not None:
        _params["$skipToken"] = _SERIALIZER.query("skip_token", skip_token, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_request(
    resource_group_name: str, job_name: str, subscription_id: str, *, expand: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if expand is not None:
        _params["$expand"] = _SERIALIZER.query("expand", expand, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_request(resource_group_name: str, job_name: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_request(resource_group_name: str, job_name: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, headers=_headers, **kwargs)


def build_update_request(
    resource_group_name: str, job_name: str, subscription_id: str, *, if_match: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if if_match is not None:
        _headers["If-Match"] = _SERIALIZER.header("if_match", if_match, "str")
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PATCH", url=_url, params=_params, headers=_headers, **kwargs)


def build_book_shipment_pick_up_request(
    resource_group_name: str, job_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}/bookShipmentPickUp",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_cancel_request(resource_group_name: str, job_name: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}/cancel",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_credentials_request(
    resource_group_name: str, job_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBox/jobs/{jobName}/listCredentials",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "jobName": _SERIALIZER.url("job_name", job_name, "str", max_length=24, min_length=3, pattern=r"^[-\w\.]+$"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class JobsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.databox.DataBoxManagementClient`'s
        :attr:`jobs` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, skip_token: Optional[str] = None, **kwargs: Any) -> Iterable["_models.JobResource"]:
        """Lists all the jobs available under the subscription.

        :param skip_token: $skipToken is supported on Get list of jobs, which provides the next page in
         the list of jobs. Default value is None.
        :type skip_token: str
        :return: An iterator like instance of either JobResource or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.JobResourceList] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    skip_token=skip_token,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("JobResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @overload
    def mark_devices_shipped(
        self,
        job_name: str,
        resource_group_name: str,
        mark_devices_shipped_request: _models.MarkDevicesShippedRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Request to mark devices for a given job as shipped.

        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param mark_devices_shipped_request: Mark Devices Shipped Request. Required.
        :type mark_devices_shipped_request: ~azure.mgmt.databox.models.MarkDevicesShippedRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def mark_devices_shipped(
        self,
        job_name: str,
        resource_group_name: str,
        mark_devices_shipped_request: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Request to mark devices for a given job as shipped.

        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param mark_devices_shipped_request: Mark Devices Shipped Request. Required.
        :type mark_devices_shipped_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def mark_devices_shipped(  # pylint: disable=inconsistent-return-statements
        self,
        job_name: str,
        resource_group_name: str,
        mark_devices_shipped_request: Union[_models.MarkDevicesShippedRequest, IO[bytes]],
        **kwargs: Any
    ) -> None:
        """Request to mark devices for a given job as shipped.

        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param mark_devices_shipped_request: Mark Devices Shipped Request. Is either a
         MarkDevicesShippedRequest type or a IO[bytes] type. Required.
        :type mark_devices_shipped_request: ~azure.mgmt.databox.models.MarkDevicesShippedRequest or
         IO[bytes]
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(mark_devices_shipped_request, (IOBase, bytes)):
            _content = mark_devices_shipped_request
        else:
            _json = self._serialize.body(mark_devices_shipped_request, "MarkDevicesShippedRequest")

        _request = build_mark_devices_shipped_request(
            job_name=job_name,
            resource_group_name=resource_group_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def list_by_resource_group(
        self, resource_group_name: str, skip_token: Optional[str] = None, **kwargs: Any
    ) -> Iterable["_models.JobResource"]:
        """Lists all the jobs available under the given resource group.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param skip_token: $skipToken is supported on Get list of jobs, which provides the next page in
         the list of jobs. Default value is None.
        :type skip_token: str
        :return: An iterator like instance of either JobResource or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.JobResourceList] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_by_resource_group_request(
                    resource_group_name=resource_group_name,
                    subscription_id=self._config.subscription_id,
                    skip_token=skip_token,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("JobResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def get(
        self, resource_group_name: str, job_name: str, expand: Optional[str] = None, **kwargs: Any
    ) -> _models.JobResource:
        """Gets information about the specified job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param expand: $expand is supported on details parameter for job, which provides details on the
         job stages. Default value is None.
        :type expand: str
        :return: JobResource or the result of cls(response)
        :rtype: ~azure.mgmt.databox.models.JobResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.JobResource] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            expand=expand,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("JobResource", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    def _create_initial(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource: Union[_models.JobResource, IO[bytes]],
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(job_resource, (IOBase, bytes)):
            _content = job_resource
        else:
            _json = self._serialize.body(job_resource, "JobResource")

        _request = build_create_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_create(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource: _models.JobResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Creates a new job with the specified parameters. Existing job cannot be updated with this API
        and should instead be updated with the Update job API.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource: Job details from request body. Required.
        :type job_resource: ~azure.mgmt.databox.models.JobResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_create(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Creates a new job with the specified parameters. Existing job cannot be updated with this API
        and should instead be updated with the Update job API.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource: Job details from request body. Required.
        :type job_resource: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_create(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource: Union[_models.JobResource, IO[bytes]],
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Creates a new job with the specified parameters. Existing job cannot be updated with this API
        and should instead be updated with the Update job API.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource: Job details from request body. Is either a JobResource type or a IO[bytes]
         type. Required.
        :type job_resource: ~azure.mgmt.databox.models.JobResource or IO[bytes]
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.JobResource] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._create_initial(
                resource_group_name=resource_group_name,
                job_name=job_name,
                job_resource=job_resource,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("JobResource", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[_models.JobResource].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[_models.JobResource](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    def _delete_initial(self, resource_group_name: str, job_name: str, **kwargs: Any) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def begin_delete(self, resource_group_name: str, job_name: str, **kwargs: Any) -> LROPoller[None]:
        """Deletes a job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._delete_initial(
                resource_group_name=resource_group_name,
                job_name=job_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    def _update_initial(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource_update_parameter: Union[_models.JobResourceUpdateParameter, IO[bytes]],
        if_match: Optional[str] = None,
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(job_resource_update_parameter, (IOBase, bytes)):
            _content = job_resource_update_parameter
        else:
            _json = self._serialize.body(job_resource_update_parameter, "JobResourceUpdateParameter")

        _request = build_update_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            if_match=if_match,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_update(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource_update_parameter: _models.JobResourceUpdateParameter,
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Updates the properties of an existing job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource_update_parameter: Job update parameters from request body. Required.
        :type job_resource_update_parameter: ~azure.mgmt.databox.models.JobResourceUpdateParameter
        :param if_match: Defines the If-Match condition. The patch will be performed only if the ETag
         of the job on the server matches this value. Default value is None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_update(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource_update_parameter: IO[bytes],
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Updates the properties of an existing job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource_update_parameter: Job update parameters from request body. Required.
        :type job_resource_update_parameter: IO[bytes]
        :param if_match: Defines the If-Match condition. The patch will be performed only if the ETag
         of the job on the server matches this value. Default value is None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_update(
        self,
        resource_group_name: str,
        job_name: str,
        job_resource_update_parameter: Union[_models.JobResourceUpdateParameter, IO[bytes]],
        if_match: Optional[str] = None,
        **kwargs: Any
    ) -> LROPoller[_models.JobResource]:
        """Updates the properties of an existing job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param job_resource_update_parameter: Job update parameters from request body. Is either a
         JobResourceUpdateParameter type or a IO[bytes] type. Required.
        :type job_resource_update_parameter: ~azure.mgmt.databox.models.JobResourceUpdateParameter or
         IO[bytes]
        :param if_match: Defines the If-Match condition. The patch will be performed only if the ETag
         of the job on the server matches this value. Default value is None.
        :type if_match: str
        :return: An instance of LROPoller that returns either JobResource or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.databox.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.JobResource] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._update_initial(
                resource_group_name=resource_group_name,
                job_name=job_name,
                job_resource_update_parameter=job_resource_update_parameter,
                if_match=if_match,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("JobResource", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[_models.JobResource].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[_models.JobResource](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    @overload
    def book_shipment_pick_up(
        self,
        resource_group_name: str,
        job_name: str,
        shipment_pick_up_request: _models.ShipmentPickUpRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ShipmentPickUpResponse:
        """Book shipment pick up.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param shipment_pick_up_request: Details of shipment pick up request. Required.
        :type shipment_pick_up_request: ~azure.mgmt.databox.models.ShipmentPickUpRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ShipmentPickUpResponse or the result of cls(response)
        :rtype: ~azure.mgmt.databox.models.ShipmentPickUpResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def book_shipment_pick_up(
        self,
        resource_group_name: str,
        job_name: str,
        shipment_pick_up_request: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ShipmentPickUpResponse:
        """Book shipment pick up.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param shipment_pick_up_request: Details of shipment pick up request. Required.
        :type shipment_pick_up_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ShipmentPickUpResponse or the result of cls(response)
        :rtype: ~azure.mgmt.databox.models.ShipmentPickUpResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def book_shipment_pick_up(
        self,
        resource_group_name: str,
        job_name: str,
        shipment_pick_up_request: Union[_models.ShipmentPickUpRequest, IO[bytes]],
        **kwargs: Any
    ) -> _models.ShipmentPickUpResponse:
        """Book shipment pick up.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param shipment_pick_up_request: Details of shipment pick up request. Is either a
         ShipmentPickUpRequest type or a IO[bytes] type. Required.
        :type shipment_pick_up_request: ~azure.mgmt.databox.models.ShipmentPickUpRequest or IO[bytes]
        :return: ShipmentPickUpResponse or the result of cls(response)
        :rtype: ~azure.mgmt.databox.models.ShipmentPickUpResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ShipmentPickUpResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(shipment_pick_up_request, (IOBase, bytes)):
            _content = shipment_pick_up_request
        else:
            _json = self._serialize.body(shipment_pick_up_request, "ShipmentPickUpRequest")

        _request = build_book_shipment_pick_up_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ShipmentPickUpResponse", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def cancel(
        self,
        resource_group_name: str,
        job_name: str,
        cancellation_reason: _models.CancellationReason,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """CancelJob.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param cancellation_reason: Reason for cancellation. Required.
        :type cancellation_reason: ~azure.mgmt.databox.models.CancellationReason
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def cancel(
        self,
        resource_group_name: str,
        job_name: str,
        cancellation_reason: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """CancelJob.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param cancellation_reason: Reason for cancellation. Required.
        :type cancellation_reason: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def cancel(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        job_name: str,
        cancellation_reason: Union[_models.CancellationReason, IO[bytes]],
        **kwargs: Any
    ) -> None:
        """CancelJob.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :param cancellation_reason: Reason for cancellation. Is either a CancellationReason type or a
         IO[bytes] type. Required.
        :type cancellation_reason: ~azure.mgmt.databox.models.CancellationReason or IO[bytes]
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(cancellation_reason, (IOBase, bytes)):
            _content = cancellation_reason
        else:
            _json = self._serialize.body(cancellation_reason, "CancellationReason")

        _request = build_cancel_request(
            resource_group_name=resource_group_name,
            job_name=job_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def list_credentials(
        self, resource_group_name: str, job_name: str, **kwargs: Any
    ) -> Iterable["_models.UnencryptedCredentials"]:
        """This method gets the unencrypted secrets related to the job.

        :param resource_group_name: The Resource Group Name. Required.
        :type resource_group_name: str
        :param job_name: The name of the job Resource within the specified resource group. job names
         must be between 3 and 24 characters in length and use any alphanumeric and underscore only.
         Required.
        :type job_name: str
        :return: An iterator like instance of either UnencryptedCredentials or the result of
         cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.databox.models.UnencryptedCredentials]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.UnencryptedCredentialsList] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_credentials_request(
                    resource_group_name=resource_group_name,
                    job_name=job_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("UnencryptedCredentialsList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ApiError, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics import TextAnalysisClient
from azure.ai.textanalytics.models import (
    MultiLanguageTextInput,
    MultiLanguageInput,
    TextSentimentAnalysisInput,
    AnalyzeTextSentimentResult,
    SentimentActionResult,
    SentenceSentiment,
)

TextAnalysisPreparer = functools.partial(
    EnvironmentVariableLoader,
    "text_analysis",
    text_analysis_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    text_analysis_key="fake_key",
)


class TestTextAnalysis(AzureRecordedTestCase):
    def create_client(self, endpoint: str, key: str) -> TextAnalysisClient:
        return TextAnalysisClient(endpoint, AzureKeyCredential(key))


class TestTextAnalysisCase(TestTextAnalysis):
    @TextAnalysisPreparer()
    @recorded_by_proxy
    def test_text_sentiment(self, text_analysis_endpoint, text_analysis_key):
        client = self.create_client(text_analysis_endpoint, text_analysis_key)

        text_a = (
            "The food and service were unacceptable, but the concierge were nice. After talking to them about the "
            "quality of the food and the process to get room service they refunded the money we spent at the "
            "restaurant and gave us a voucher for nearby restaurants."
        )

        body = TextSentimentAnalysisInput(
            text_input=MultiLanguageTextInput(
                multi_language_inputs=[MultiLanguageInput(id="A", text=text_a, language="en")]
            )
        )

        # Sync (non-LRO) call
        result = client.analyze_text(body=body)

        assert result is not None
        assert isinstance(result, AnalyzeTextSentimentResult)

        assert result.results is not None
        assert result.results.documents is not None

        for doc in result.results.documents:
            assert isinstance(doc, SentimentActionResult)
            assert doc.id is not None
            assert doc.sentiment is not None
            assert doc.confidence_scores is not None
            assert doc.confidence_scores.positive is not None
            assert doc.confidence_scores.neutral is not None
            assert doc.confidence_scores.negative is not None
            assert doc.sentences is not None

            for sentence in doc.sentences:
                assert isinstance(sentence, SentenceSentiment)
                assert sentence.text is not None
                assert sentence.sentiment is not None
                assert sentence.confidence_scores is not None
                assert sentence.confidence_scores.positive is not None
                assert sentence.confidence_scores.neutral is not None
                assert sentence.confidence_scores.negative is not None
                assert sentence.offset is not None
                assert sentence.length is not None



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.billingbenefits.aio import BillingBenefitsMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBillingBenefitsMgmtDiscountsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BillingBenefitsMgmtClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_begin_create(self, resource_group):
        response = await (
            await self.client.discounts.begin_create(
                resource_group_name=resource_group.name,
                discount_name="str",
                body={
                    "location": "str",
                    "etag": "str",
                    "id": "str",
                    "identity": {
                        "type": "str",
                        "principalId": "str",
                        "tenantId": "str",
                        "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                    },
                    "kind": "str",
                    "managedBy": "str",
                    "name": "str",
                    "plan": {
                        "name": "str",
                        "product": "str",
                        "publisher": "str",
                        "promotionCode": "str",
                        "version": "str",
                    },
                    "properties": "discount_properties",
                    "sku": {"name": "str", "capacity": 0, "family": "str", "size": "str", "tier": "str"},
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_begin_delete(self, resource_group):
        response = await (
            await self.client.discounts.begin_delete(
                resource_group_name=resource_group.name,
                discount_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_resource_group_list(self, resource_group):
        response = self.client.discounts.resource_group_list(
            resource_group_name=resource_group.name,
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_subscription_list(self, resource_group):
        response = self.client.discounts.subscription_list()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_begin_cancel(self, resource_group):
        response = await (
            await self.client.discounts.begin_cancel(
                resource_group_name=resource_group.name,
                discount_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discounts_scope_list(self, resource_group):
        response = self.client.discounts.scope_list(
            scope="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.datafactory import DataFactoryManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-datafactory
# USAGE
    python triggers_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DataFactoryManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.triggers.create_or_update(
        resource_group_name="exampleResourceGroup",
        factory_name="exampleFactoryName",
        trigger_name="exampleTrigger",
        trigger={
            "properties": {
                "description": "Example description",
                "pipelines": [
                    {
                        "parameters": {"OutputBlobNameList": ["exampleoutput.csv"]},
                        "pipelineReference": {"referenceName": "examplePipeline", "type": "PipelineReference"},
                    }
                ],
                "type": "ScheduleTrigger",
                "typeProperties": {
                    "recurrence": {
                        "endTime": "2018-06-16T00:55:14.905167Z",
                        "frequency": "Minute",
                        "interval": 4,
                        "startTime": "2018-06-16T00:39:14.905167Z",
                        "timeZone": "UTC",
                    }
                },
            }
        },
    )
    print(response)


# x-ms-original-file: 2018-06-01/Triggers_Update.json
if __name__ == "__main__":
    main()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------

from azure.core.pipeline import PipelineRequest
from azure.core.pipeline.policies import UserAgentPolicy
from azure.core.pipeline.policies._universal import HTTPRequestType

from azure.cosmos._utils import get_user_agent_features


class CosmosUserAgentPolicy(UserAgentPolicy):
    """Custom user agent policy for Cosmos DB that appends feature flags to the user agent string.

    This policy extends the standard UserAgentPolicy to include Cosmos-specific feature flags
    (e.g., circuit breaker, per-partition automatic failover) in the user agent header for
    debugging and telemetry purposes.
    """

    def on_request(self, request: PipelineRequest[HTTPRequestType]) -> None:
        """Modifies the User-Agent header before the request is sent.

        :param request: The PipelineRequest object
        :type request: ~azure.core.pipeline.PipelineRequest
        """
        options_dict = request.context.options
        # Add relevant enabled features to user agent for debugging
        if "global_endpoint_manager" in options_dict:
            global_endpoint_manager = options_dict["global_endpoint_manager"]
            user_agent_features = get_user_agent_features(global_endpoint_manager)
            if len(user_agent_features) > 0:
                user_agent = "{} {}".format(self._user_agent, user_agent_features)
                options_dict["user_agent"] = user_agent
                options_dict["user_agent_overwrite"] = True
        super().on_request(request)



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import numbers
import json
from typing import List
from opentelemetry.sdk.trace import Span


class GenAiTraceVerifier:

    def check_span_attributes(self, span, attributes):
        assert "https://opentelemetry.io/schemas/1.34.0" == span.instrumentation_scope.schema_url

        # Convert the list of tuples to a dictionary for easier lookup
        attribute_dict = dict(attributes)
        attribute_dict["az.namespace"] = "Microsoft.CognitiveServices"

        # First, check that all expected attributes are present in the span
        for expected_attribute_name in attribute_dict.keys():
            if expected_attribute_name not in span.attributes:
                raise AssertionError(
                    f"Expected attribute '{expected_attribute_name}' not found in span. "
                    f"Span has: {list(span.attributes.keys())}"
                )

        # Then, check that all attributes in the span are expected and have correct values
        for attribute_name in span.attributes.keys():
            # Check if the attribute name exists in the input attributes
            if attribute_name not in attribute_dict:
                raise AssertionError("Attribute name " + attribute_name + " not in attribute dictionary")

            attribute_value = attribute_dict[attribute_name]
            if isinstance(attribute_value, list):
                # Check if the attribute value in the span matches the provided list
                if span.attributes[attribute_name] != attribute_value:
                    raise AssertionError(
                        "Attribute value list "
                        + str(span.attributes[attribute_name])
                        + " does not match with "
                        + str(attribute_value)
                    )
            elif isinstance(attribute_value, tuple):
                # Check if the attribute value in the span matches the provided list
                if span.attributes[attribute_name] != attribute_value:
                    raise AssertionError(
                        "Attribute value tuple "
                        + str(span.attributes[attribute_name])
                        + " does not match with "
                        + str(attribute_value)
                    )
            else:
                # Check if the attribute value matches the provided value
                if attribute_value == "+":
                    if not isinstance(span.attributes[attribute_name], numbers.Number):
                        raise AssertionError(
                            "Attribute value " + str(span.attributes[attribute_name]) + " is not a number"
                        )
                    if span.attributes[attribute_name] < 0:
                        raise AssertionError("Attribute value " + str(span.attributes[attribute_name]) + " is negative")
                elif attribute_value not in ("", span.attributes[attribute_name]):
                    raise AssertionError(
                        "Attribute value "
                        + str(span.attributes[attribute_name])
                        + " does not match with "
                        + str(attribute_value)
                    )
                # Check if the attribute value in the span is not empty when the provided value is ""
                elif attribute_value == "" and not span.attributes[attribute_name]:
                    raise AssertionError("Expected non-empty attribute value")

        return True

    def check_decorator_span_attributes(self, span: Span, attributes: List[tuple]) -> bool:
        # Convert the list of tuples to a dictionary for easier lookup
        attribute_dict = dict(attributes)

        # Ensure all required attributes are present in the span
        for attribute_name in attribute_dict.keys():
            if attribute_name not in span.attributes:
                raise AssertionError("Required attribute name " + attribute_name + " not found in span attributes")

        for attribute_name in span.attributes.keys():
            # Check if the attribute name exists in the input attributes
            if attribute_name not in attribute_dict:
                raise AssertionError("Attribute name " + attribute_name + " not in attribute dictionary")

            attribute_value = attribute_dict[attribute_name]
            span_value = span.attributes[attribute_name]

            if isinstance(attribute_value, (list, tuple)):
                # Convert both to lists for comparison
                if list(span_value) != list(attribute_value):
                    raise AssertionError(
                        "Attribute value list/tuple " + str(span_value) + " does not match with " + str(attribute_value)
                    )
            elif isinstance(attribute_value, dict):
                # Check if both are dictionaries and compare them
                if not isinstance(span_value, dict) or span_value != attribute_value:
                    raise AssertionError(
                        "Attribute value dict " + str(span_value) + " does not match with " + str(attribute_value)
                    )
            else:
                # Check if the attribute value matches the provided value
                if attribute_value == "+":
                    if not isinstance(span_value, numbers.Number):
                        raise AssertionError("Attribute value " + str(span_value) + " is not a number")
                    if span_value < 0:
                        raise AssertionError("Attribute value " + str(span_value) + " is negative")
                elif attribute_value not in ("", span_value):
                    raise AssertionError(
                        "Attribute value " + str(span_value) + " does not match with " + str(attribute_value)
                    )
                # Check if the attribute value in the span is not empty when the provided value is ""
                elif attribute_value == "" and not span_value:
                    raise AssertionError("Expected non-empty attribute value")

        return True

    def is_valid_json(self, my_string):
        try:
            json.loads(my_string)
        except ValueError:
            return False
        except TypeError:
            return False
        return True

    def check_json_string(self, expected_json, actual_json):
        assert self.is_valid_json(expected_json) and self.is_valid_json(actual_json)
        expected_obj = json.loads(expected_json)
        actual_obj = json.loads(actual_json)

        # Handle both dict and list (array) formats
        if isinstance(expected_obj, list) and isinstance(actual_obj, list):
            return self.check_event_lists(expected_obj, actual_obj)
        if isinstance(expected_obj, dict) and isinstance(actual_obj, dict):
            return self.check_event_attributes(expected_obj, actual_obj)
        raise AssertionError(
            f"check_json_string: type mismatch - expected {type(expected_obj).__name__}, got {type(actual_obj).__name__}"
        )

    def check_event_lists(self, expected_list, actual_list):
        """Check if two lists match, handling nested dicts/lists."""
        if len(expected_list) != len(actual_list):
            raise AssertionError(
                f"check_event_lists: list lengths do not match: {len(expected_list)} != {len(actual_list)}"
            )

        for i, (expected_item, actual_item) in enumerate(zip(expected_list, actual_list)):
            if isinstance(expected_item, dict) and isinstance(actual_item, dict):
                self.check_event_attributes(expected_item, actual_item)
            elif isinstance(expected_item, list) and isinstance(actual_item, list):
                self.check_event_lists(expected_item, actual_item)
            else:
                # For primitive values, do direct comparison
                if expected_item != actual_item:
                    raise AssertionError(
                        f"check_event_lists: item at index {i} does not match: {expected_item} != {actual_item}"
                    )
        return True

    def check_event_attributes(self, expected_dict, actual_dict):
        if set(expected_dict.keys()) != set(actual_dict.keys()):
            if isinstance(expected_dict, dict):
                expected_val = json.dumps(expected_dict)
            else:
                expected_val = expected_dict
            if isinstance(actual_dict, dict):
                actual_val = json.dumps(actual_dict)
            else:
                actual_val = actual_dict
            raise AssertionError(
                f"check_event_attributes: keys do not match: {set(expected_dict.keys())} != {set(actual_dict.keys())}. The actual dictionaries: {expected_val} != {actual_val}"
            )
        for key, expected_val in expected_dict.items():
            if key not in actual_dict:
                raise AssertionError(f"check_event_attributes: key {key} not found in actuals")
            actual_val = actual_dict[key]

            if self.is_valid_json(expected_val):
                if not self.is_valid_json(actual_val):
                    raise AssertionError(f"check_event_attributes: actual_val for {key} is not valid json")
                self.check_json_string(expected_val, actual_val)
            elif isinstance(expected_val, dict):
                if not isinstance(actual_val, dict):
                    raise AssertionError(f"check_event_attributes: actual_val for {key} is not dict")
                self.check_event_attributes(expected_val, actual_val)
            elif isinstance(expected_val, list):
                if not isinstance(actual_val, list):
                    raise AssertionError(f"check_event_attributes: actual_val for {key} is not list")
                self.check_event_lists(expected_val, actual_val)
            elif isinstance(expected_val, str) and expected_val == "*":
                if actual_val == "":
                    raise AssertionError(f"check_event_attributes: actual_val for {key} is empty")
            elif isinstance(expected_val, str) and expected_val == "+":
                assert isinstance(actual_val, numbers.Number), f"The {key} is not a number."
                assert actual_val > 0, f"The {key} is <0 {actual_val}"
            elif expected_val != actual_val:
                if isinstance(expected_val, dict):
                    expected_val = json.dumps(expected_val)
                if isinstance(actual_val, dict):
                    actual_val = json.dumps(actual_val)
                raise AssertionError(
                    f"check_event_attributes: values do not match for key {key}: {expected_val} != {actual_val}"
                )

    def check_span_events(self, span, expected_events):
        print("Checking span: " + span.name)
        span_events = list(span.events)  # Create a list of events from the span

        for expected_event in expected_events:
            for actual_event in span_events:
                if expected_event["name"] == actual_event.name:
                    self.check_event_attributes(expected_event["attributes"], actual_event.attributes._dict)
                    span_events.remove(actual_event)  # Remove the matched event from the span_events
                    break
            else:
                raise AssertionError("check_span_events: event not found")

        if len(span_events) > 0:  # If there are any additional events in the span_events
            unexpected_event_names = [event.name for event in span_events]
            raise AssertionError(f"check_span_events: unexpected event(s) found: {unexpected_event_names}")

        return True



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.costmanagement import CostManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCostManagementAlertsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CostManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_alerts_get(self, resource_group):
        response = self.client.alerts.get(
            scope="str",
            alert_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_alerts_dismiss(self, resource_group):
        response = self.client.alerts.dismiss(
            scope="str",
            alert_id="str",
            parameters={
                "properties": {
                    "closeTime": "str",
                    "costEntityId": "str",
                    "creationTime": "str",
                    "definition": {"category": "str", "criteria": "str", "type": "str"},
                    "description": "str",
                    "details": {
                        "amount": 0.0,
                        "companyName": "str",
                        "contactEmails": ["str"],
                        "contactGroups": ["str"],
                        "contactRoles": ["str"],
                        "currentSpend": 0.0,
                        "departmentName": "str",
                        "enrollmentEndDate": "str",
                        "enrollmentNumber": "str",
                        "enrollmentStartDate": "str",
                        "invoicingThreshold": 0.0,
                        "meterFilter": [{}],
                        "operator": "str",
                        "overridingAlert": "str",
                        "periodStartDate": "str",
                        "resourceFilter": [{}],
                        "resourceGroupFilter": [{}],
                        "tagFilter": {},
                        "threshold": 0.0,
                        "timeGrainType": "str",
                        "triggeredBy": "str",
                        "unit": "str",
                    },
                    "modificationTime": "str",
                    "source": "str",
                    "status": "str",
                    "statusModificationTime": "str",
                    "statusModificationUserName": "str",
                }
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_alerts_list(self, resource_group):
        response = self.client.alerts.list(
            scope="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_alerts_list_external(self, resource_group):
        response = self.client.alerts.list_external(
            external_cloud_provider_type="str",
            external_cloud_provider_id="str",
        )

        # please add some check logic here by yourself
        # ...



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import functools
import pytest
import conftest
from devtools_testutils import EnvironmentVariableLoader, recorded_by_proxy
from testcase import AppConfigTestCase, set_test_settings
from test_constants import APPCONFIGURATION_ENDPOINT_STRING
from azure.appconfiguration.provider._models import SettingSelector
from azure.appconfiguration.provider._constants import NULL_CHAR, FEATURE_MANAGEMENT_KEY, FEATURE_FLAG_KEY
from azure.appconfiguration.provider import WatchKey
from azure.appconfiguration import (
    ConfigurationSetting,
    FeatureFlagConfigurationSetting,
)

AppConfigProviderPreparer = functools.partial(
    EnvironmentVariableLoader,
    "appconfiguration",
    appconfiguration_endpoint_string=APPCONFIGURATION_ENDPOINT_STRING,
)


class TestSnapshotSupport:
    """Tests for snapshot functionality in SettingSelector."""

    def test_setting_selector_with_snapshot_name(self):
        """Test SettingSelector with snapshot_name parameter."""
        selector = SettingSelector(snapshot_name="my-snapshot")
        assert selector.snapshot_name == "my-snapshot"
        assert selector.key_filter is None
        assert selector.label_filter == NULL_CHAR
        assert selector.tag_filters is None

    def test_setting_selector_snapshot_name_with_key_filter_raises_error(self):
        """Test that SettingSelector raises ValueError when both snapshot_name and key_filter are provided."""
        with pytest.raises(
            ValueError,
            match=r"Cannot specify both snapshot_name and key_filter\. "
            r"When using snapshots, all other filtering parameters are ignored\.",
        ):
            SettingSelector(snapshot_name="my-snapshot", key_filter="*")

    def test_setting_selector_snapshot_name_with_label_filter_raises_error(self):
        """Test that SettingSelector raises ValueError when both snapshot_name and label_filter are provided."""
        with pytest.raises(
            ValueError,
            match=r"Cannot specify both snapshot_name and label_filter\. "
            r"When using snapshots, all other filtering parameters are ignored\.",
        ):
            SettingSelector(snapshot_name="my-snapshot", label_filter="prod")

    def test_setting_selector_snapshot_name_with_tag_filters_raises_error(self):
        """Test that SettingSelector raises ValueError when both snapshot_name and tag_filters are provided."""
        with pytest.raises(
            ValueError,
            match=r"Cannot specify both snapshot_name and tag_filters\. "
            r"When using snapshots, all other filtering parameters are ignored\.",
        ):
            SettingSelector(snapshot_name="my-snapshot", tag_filters=["env=prod"])

    def test_setting_selector_requires_key_filter_or_snapshot_name(self):
        """Test that SettingSelector requires either key_filter or snapshot_name."""
        with pytest.raises(ValueError, match=r"Either key_filter or snapshot_name must be specified\."):
            SettingSelector()

    def test_setting_selector_valid_combinations(self):
        """Test valid combinations of SettingSelector parameters."""
        # Valid: key_filter only
        selector = SettingSelector(key_filter="*")
        assert selector.key_filter == "*"
        assert selector.snapshot_name is None

        # Valid: snapshot_name only
        selector = SettingSelector(snapshot_name="my-snapshot")
        assert selector.snapshot_name == "my-snapshot"
        assert selector.key_filter is None

        # Valid: key_filter with other filters (no snapshot_name)
        selector = SettingSelector(key_filter="*", label_filter="prod", tag_filters=["env=prod"])
        assert selector.key_filter == "*"
        assert selector.label_filter == "prod"
        assert selector.tag_filters == ["env=prod"]
        assert selector.snapshot_name is None


class TestSnapshotProviderIntegration(AppConfigTestCase):
    """Integration tests for snapshot functionality with recorded tests."""

    def test_setting_selector_multiple_combinations(self):
        """Test using multiple selectors with mix of snapshot and regular selectors."""
        # This should be valid - mixing snapshot and regular selectors
        selectors = [
            SettingSelector(snapshot_name="config-snapshot"),  # Load from snapshot
            SettingSelector(key_filter="runtime.*"),  # Load runtime configs from current state
        ]

        # Validate each selector individually
        assert selectors[0].snapshot_name == "config-snapshot"
        assert selectors[0].key_filter is None

        assert selectors[1].key_filter == "runtime.*"
        assert selectors[1].snapshot_name is None

    def test_snapshot_with_different_names(self):
        """Test snapshot selectors with different snapshot names."""
        # Different snapshot names should be valid
        snapshot1 = SettingSelector(snapshot_name="prod-config-v1.0")
        snapshot2 = SettingSelector(snapshot_name="staging-config-v2.1")

        assert snapshot1.snapshot_name == "prod-config-v1.0"
        assert snapshot2.snapshot_name == "staging-config-v2.1"

        # Both should have no other filters
        for selector in [snapshot1, snapshot2]:
            assert selector.key_filter is None
            assert selector.label_filter == NULL_CHAR
            assert selector.tag_filters is None

    @AppConfigProviderPreparer()
    @recorded_by_proxy
    def test_load_provider_with_snapshot_not_found(self, appconfiguration_endpoint_string):
        """Test loading provider with a non-existent snapshot returns error."""
        # Try to load from a non-existent snapshot
        provider = self.create_client(
            endpoint=appconfiguration_endpoint_string,
            selects=[SettingSelector(snapshot_name="non-existent-snapshot")],
        )
        assert len(provider._dict) == 0, "Provider should have no settings when snapshot is not found"

    @AppConfigProviderPreparer()
    @recorded_by_proxy
    def test_load_provider_with_regular_selectors(self, appconfiguration_endpoint_string):
        """Test loading provider with regular selectors works (baseline test)."""
        # This should work - regular selector loading
        provider = self.create_client(
            endpoint=appconfiguration_endpoint_string,
            selects=[SettingSelector(key_filter="message")],  # Regular selector
        )

        # Verify we can access the configuration (message is set up by setup_configs)
        assert "message" in provider

    @pytest.mark.live_test_only  # Needed to fix an azure core dependency compatibility issue
    @AppConfigProviderPreparer()
    @recorded_by_proxy
    def test_create_snapshot_and_load_provider(self, appconfiguration_endpoint_string):
        """Test loading provider from a pre-created snapshot."""
        snapshot_name = conftest.snapshot_names["snapshot"]

        # Load provider using the snapshot with refresh enabled and feature flags enabled
        provider = self.create_client(
            endpoint=appconfiguration_endpoint_string,
            selects=[
                SettingSelector(snapshot_name=snapshot_name),  # Snapshot data (includes feature flags)
                SettingSelector(key_filter="refresh_test_key"),  # Non-snapshot key for refresh testing
            ],
            refresh_on=[WatchKey("refresh_test_key")],  # Watch non-snapshot key for refresh
            refresh_interval=1,  # Short refresh interval for testing
        )

        # Get the underlying SDK client from the provider
        sdk_client = provider._replica_client_manager._original_client._client

        # Verify all snapshot settings are loaded
        assert provider["snapshot_test_key1"] == "snapshot_test_value1"
        assert provider["snapshot_test_key2"] == "snapshot_test_value2"
        assert provider["snapshot_test_json"]["nested"] == "snapshot_value"
        assert provider["refresh_test_key"] == "original_refresh_value"

        # Verify that snapshot settings and refresh key are loaded
        snapshot_keys = [key for key in provider.keys() if key.startswith("snapshot_test_")]
        assert len(snapshot_keys) == 3

        # Verify feature flags from snapshots are NOT loaded as feature flags
        # (snapshots don't support feature flag loading, only regular selects do)
        assert FEATURE_MANAGEMENT_KEY not in provider, "Feature flags should not be loaded from snapshots"

        try:
            # Test snapshot immutability: modify the original settings
            modified_settings = [
                ConfigurationSetting(
                    key="snapshot_test_key1", value="MODIFIED_VALUE1", label=NULL_CHAR  # Changed value
                ),
                ConfigurationSetting(
                    key="snapshot_test_key2", value="MODIFIED_VALUE2", label=NULL_CHAR  # Changed value
                ),
                ConfigurationSetting(
                    key="snapshot_test_json",
                    value='{"nested": "MODIFIED_VALUE"}',  # Changed nested value
                    label=NULL_CHAR,
                    content_type="application/json",
                ),
                ConfigurationSetting(
                    key="refresh_test_key",
                    value="updated_refresh_value",  # Changed value to trigger refresh
                    label=NULL_CHAR,
                ),
            ]

            # Update the original settings with new values and add a new key
            new_key = ConfigurationSetting(key="new_key_added_after_load", value="new_value", label=NULL_CHAR)
            set_test_settings(sdk_client, modified_settings + [new_key])

            # Expire the refresh timer so refresh will check for changes
            provider._refresh_timer._next_refresh_time = 0

            # Refresh the existing provider (snapshots should remain immutable, but non-snapshot keys should update)
            provider.refresh()

            # Verify the snapshot still contains the original values after refresh (immutability)
            assert provider["snapshot_test_key1"] == "snapshot_test_value1"  # Original value
            assert provider["snapshot_test_key2"] == "snapshot_test_value2"  # Original value
            assert provider["snapshot_test_json"]["nested"] == "snapshot_value"  # Original value

            # Verify the non-snapshot key was updated during refresh
            assert provider["refresh_test_key"] == "updated_refresh_value"  # Updated value

            # Verify new keys are NOT added during refresh (only watched keys trigger full reload)
            assert "new_key_added_after_load" not in provider  # New key should not be loaded

            # Verify that loading without snapshot gets the modified values
            provider_current = self.create_client(
                endpoint=appconfiguration_endpoint_string,
                selects=[SettingSelector(key_filter="snapshot_test_*")],
            )

            # Current values should be the modified ones
            assert provider_current["snapshot_test_key1"] == "MODIFIED_VALUE1"  # Modified value
            assert provider_current["snapshot_test_key2"] == "MODIFIED_VALUE2"  # Modified value
            assert provider_current["snapshot_test_json"]["nested"] == "MODIFIED_VALUE"  # Modified value
        finally:
            # Restore modified settings to original values for other tests
            restored_settings = [
                ConfigurationSetting(key="snapshot_test_key1", value="snapshot_test_value1", label=NULL_CHAR),
                ConfigurationSetting(key="snapshot_test_key2", value="snapshot_test_value2", label=NULL_CHAR),
                ConfigurationSetting(
                    key="snapshot_test_json",
                    value='{"nested": "snapshot_value"}',
                    label=NULL_CHAR,
                    content_type="application/json",
                ),
                ConfigurationSetting(key="refresh_test_key", value="original_refresh_value", label=NULL_CHAR),
            ]
            set_test_settings(sdk_client, restored_settings)

    @pytest.mark.live_test_only  # Needed to fix an azure core dependency compatibility issue
    @AppConfigProviderPreparer()
    @recorded_by_proxy
    def test_create_snapshot_and_load_provider_with_feature_flags(self, appconfiguration_endpoint_string):
        """Test loading provider with feature flags from a pre-created snapshot."""
        snapshot_name = conftest.snapshot_names["ff_snapshot"]

        # Load provider using snapshot for config settings and regular selectors for feature flags
        provider = self.create_client(
            endpoint=appconfiguration_endpoint_string,
            feature_flag_enabled=True,  # Enable feature flags
            feature_flag_selectors=[
                SettingSelector(snapshot_name=snapshot_name),  # Load feature flags from snapshot
            ],
        )

        # Get the underlying SDK client from the provider
        sdk_client = provider._replica_client_manager._original_client._client

        # Verify snapshot configuration settings are loaded
        assert provider["ff_snapshot_test_key1"] == "ff_snapshot_test_value1"
        assert provider["ff_snapshot_test_key2"] == "ff_snapshot_test_value2"

        # Verify feature flags loaded via regular selectors ARE loaded
        feature_flags = provider.get(FEATURE_MANAGEMENT_KEY, {}).get(FEATURE_FLAG_KEY, [])
        feature_flag_ids = {ff["id"]: ff["enabled"] for ff in feature_flags}

        # Regular feature flags should be loaded
        assert "RegularFeature" not in feature_flag_ids, "RegularFeature should not be loaded via regular selector"
        assert "RegularFeatureDisabled" not in feature_flag_ids, "RegularFeatureDisabled should not be loaded"

        # Snapshot-only feature flag should be loaded as a feature flag
        assert "SnapshotOnlyFeature" in feature_flag_ids, "SnapshotOnlyFeature should be loaded as FF from snapshot"

        # Verify exactly 1 feature flag is loaded (the snapshot-only one)
        assert len(feature_flags) == 1, f"Expected 1 feature flag, got {len(feature_flags)}"

        try:
            # Modify the feature flags in the snapshot
            modified_feature_flags = [
                FeatureFlagConfigurationSetting(
                    feature_id="SnapshotOnlyFeature",
                    enabled=False,  # Changed from True to False
                    label=NULL_CHAR,
                ),
            ]

            set_test_settings(sdk_client, modified_feature_flags)

            # Load a fresh provider without snapshot to verify current feature flag values
            provider_current = self.create_client(
                endpoint=appconfiguration_endpoint_string,
                feature_flag_enabled=True,
                feature_flag_selectors=[
                    SettingSelector(snapshot_name=snapshot_name),  # Load feature flags from snapshot
                ],
            )

            # Current feature flag values should be the original ones from snapshot (immutable)
            current_feature_flags = provider_current.get(FEATURE_MANAGEMENT_KEY, {}).get(FEATURE_FLAG_KEY, [])
            current_ff_ids = {ff["id"]: ff["enabled"] for ff in current_feature_flags}
            assert current_ff_ids.get("SnapshotOnlyFeature") is True  # Original value from snapshot (not modified)
        finally:
            # Restore modified feature flag to original value for other tests
            set_test_settings(
                sdk_client,
                [FeatureFlagConfigurationSetting(feature_id="SnapshotOnlyFeature", enabled=True, label=NULL_CHAR)],
            )



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.desktopvirtualization import DesktopVirtualizationMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-desktopvirtualization
# USAGE
    python host_pools_list_registration_token_post.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DesktopVirtualizationMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="daefabc0-95b4-48b3-b645-8a753a63c4fa",
    )

    response = client.host_pools.list_registration_tokens(
        resource_group_name="resourceGroup1",
        host_pool_name="hostPool1",
    )
    print(response)


# x-ms-original-file: specification/desktopvirtualization/resource-manager/Microsoft.DesktopVirtualization/stable/2024-04-03/examples/HostPools_ListRegistrationToken_Post.json
if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_create_project.py
DESCRIPTION:
    This sample demonstrates how to create a **Text Authoring** project.
USAGE:
    python sample_create_project.py
REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_TEXT_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_TEXT_ENDPOINT
      - AZURE_TEXT_KEY
OPTIONAL ENV VARS:
    PROJECT_NAME                  # defaults to "<project-name>"
    LANGUAGE_TAG                  # defaults to "<language-tag>"
    STORAGE_INPUT_CONTAINER_NAME  # defaults to "<storage-container-name>"
    DESCRIPTION                   # defaults to "<description>"
"""

# [START text_authoring_create_project]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.textanalytics.authoring import TextAuthoringClient
from azure.ai.textanalytics.authoring.models import (
    CreateProjectOptions,
    ProjectKind,
)


def sample_text_create_project():
    # settings
    endpoint = os.environ["AZURE_TEXT_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    language_tag = os.environ.get("LANGUAGE_TAG", "<language-tag>")  # e.g., "en"
    storage_container = os.environ.get("STORAGE_INPUT_CONTAINER_NAME", "<storage-container-name>")
    description = os.environ.get("DESCRIPTION", "<description>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = TextAuthoringClient(endpoint, credential=credential)

    # build project definition (Custom Multi-Label Classification)
    body = CreateProjectOptions(
        project_kind=ProjectKind.CUSTOM_MULTI_LABEL_CLASSIFICATION,
        storage_input_container_name=storage_container,
        project_name=project_name,
        language=language_tag,
        description=description,
        multilingual=True,
    )

    # create project
    result = client.create_project(project_name=project_name, body=body)

    # print project details (direct attribute access; no getattr)
    print("=== Create Text Authoring Project Result ===")
    print(f"Project Name: {result.project_name}")
    print(f"Language: {result.language}")
    print(f"Kind: {result.project_kind}")
    print(f"Multilingual: {result.multilingual}")
    print(f"Description: {result.description}")
    print(f"Storage Input Container: {result.storage_input_container_name}")


# [END text_authoring_create_project]


def main():
    sample_text_create_project()


if __name__ == "__main__":
    main()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
from unittest import mock
import asyncio
from packaging.version import Version

from corehttp.rest import HttpRequest
from corehttp.transport.aiohttp import AioHttpTransport
from corehttp.runtime.pipeline import AsyncPipeline
from corehttp.exceptions import (
    ServiceResponseError,
    ServiceRequestError,
    ServiceRequestTimeoutError,
    ServiceResponseTimeoutError,
)

import aiohttp
import pytest


@pytest.mark.asyncio
async def test_already_close_with_with(caplog, port):
    transport = AioHttpTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    async with AsyncPipeline(transport) as pipeline:
        await pipeline.run(request)

    # This is now closed, new requests should fail
    with pytest.raises(ValueError) as err:
        await transport.send(request)
    assert "HTTP transport has already been closed." in str(err)


@pytest.mark.asyncio
async def test_already_close_manually(caplog, port):
    transport = AioHttpTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    await transport.send(request)
    await transport.close()

    # This is now closed, new requests should fail
    with pytest.raises(ValueError) as err:
        await transport.send(request)
    assert "HTTP transport has already been closed." in str(err)


@pytest.mark.asyncio
async def test_close_too_soon_works_fine(caplog, port):
    transport = AioHttpTransport()

    request = HttpRequest("GET", "http://localhost:{}/basic/string".format(port))

    await transport.close()
    result = await transport.send(request)

    assert result  # No exception is good enough here


@pytest.mark.asyncio
async def test_aiohttp_timeout_response(port):
    async with AioHttpTransport() as transport:

        request = HttpRequest("GET", f"http://localhost:{port}/basic/string")

        with mock.patch.object(
            aiohttp.ClientResponse, "start", side_effect=asyncio.TimeoutError("Too slow!")
        ) as mock_method:
            with pytest.raises(ServiceResponseTimeoutError) as err:
                await transport.send(request)

            with pytest.raises(ServiceResponseError) as err:
                await transport.send(request)

            stream_resp = HttpRequest("GET", f"http://localhost:{port}/streams/basic")
            with pytest.raises(ServiceResponseTimeoutError) as err:
                await transport.send(stream_resp, stream=True)

        stream_resp = await transport.send(stream_resp, stream=True)
        with mock.patch.object(
            aiohttp.streams.StreamReader, "read", side_effect=asyncio.TimeoutError("Too slow!")
        ) as mock_method:
            with pytest.raises(ServiceResponseTimeoutError) as err:
                await stream_resp.read()


@pytest.mark.asyncio
async def test_aiohttp_timeout_request():
    async with AioHttpTransport() as transport:
        transport.session._connector.connect = mock.Mock(side_effect=asyncio.TimeoutError("Too slow!"))

        request = HttpRequest("GET", f"http://localhost:12345/basic/string")

        # aiohttp 3.10 introduced separate connection timeout
        if Version(aiohttp.__version__) >= Version("3.10"):
            with pytest.raises(ServiceRequestTimeoutError) as err:
                await transport.send(request)

            with pytest.raises(ServiceRequestError) as err:
                await transport.send(request)

            stream_request = HttpRequest("GET", f"http://localhost:12345/streams/basic")
            with pytest.raises(ServiceRequestTimeoutError) as err:
                await transport.send(stream_request, stream=True)

        else:
            with pytest.raises(ServiceResponseTimeoutError) as err:
                await transport.send(request)

            with pytest.raises(ServiceResponseError) as err:
                await transport.send(request)

            stream_request = HttpRequest("GET", f"http://localhost:12345/streams/basic")
            with pytest.raises(ServiceResponseTimeoutError) as err:
                await transport.send(stream_request, stream=True)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Contract tests for inbound request logging middleware and handler diagnostic logging.

Validates that:
- InboundRequestLoggingMiddleware logs request start and completion at INFO.
- Status >= 400 triggers WARNING on completion.
- Correlation headers (x-request-id, x-ms-client-request-id) appear in log.
- Query strings are NOT logged (path only).
- Handler-level diagnostic logs fire at INFO for each endpoint.
- Orchestrator logs handler invocation with handler name.
"""

from __future__ import annotations

import asyncio
import json as _json
import logging
from typing import Any

import pytest

from azure.ai.agentserver.responses import ResponsesAgentServerHost

# ── Helpers ───────────────────────────────────────────────

LOGGER_NAME = "azure.ai.agentserver"

# A valid-format response ID that will never exist in state/storage.
_NONEXISTENT_ID = "caresp_00000000000000000000000000000000000000000000000000"


def _make_app(handler=None):
    """Create a host with a simple handler."""
    app = ResponsesAgentServerHost(configure_observability=None)

    @app.response_handler
    def _default_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            if False:  # pragma: no cover
                yield None

        return _events()

    if handler is not None:
        app.response_handler(handler)
    return app


class _AsgiResponse:
    def __init__(self, status_code: int, body: bytes, headers: list[tuple[bytes, bytes]]) -> None:
        self.status_code = status_code
        self.body = body
        self.headers = headers

    def json(self) -> Any:
        return _json.loads(self.body)


class _AsyncAsgiClient:
    def __init__(self, app: Any) -> None:
        self._app = app

    @staticmethod
    def _build_scope(
        method: str,
        path: str,
        body: bytes,
        headers: list[tuple[bytes, bytes]] | None = None,
    ) -> dict[str, Any]:
        hdr: list[tuple[bytes, bytes]] = list(headers or [])
        query_string = b""
        if "?" in path:
            path, qs = path.split("?", 1)
            query_string = qs.encode()
        if body:
            hdr += [
                (b"content-type", b"application/json"),
                (b"content-length", str(len(body)).encode()),
            ]
        return {
            "type": "http",
            "asgi": {"version": "3.0"},
            "http_version": "1.1",
            "method": method,
            "headers": hdr,
            "scheme": "http",
            "path": path,
            "raw_path": path.encode(),
            "query_string": query_string,
            "server": ("localhost", 80),
            "client": ("127.0.0.1", 123),
            "root_path": "",
        }

    async def request(
        self,
        method: str,
        path: str,
        *,
        json_body: dict[str, Any] | None = None,
        headers: dict[str, str] | None = None,
    ) -> _AsgiResponse:
        body = _json.dumps(json_body).encode() if json_body else b""
        raw_headers = [(k.lower().encode(), v.encode()) for k, v in headers.items()] if headers else []
        scope = self._build_scope(method, path, body, raw_headers)
        status_code: int | None = None
        response_headers: list[tuple[bytes, bytes]] = []
        body_parts: list[bytes] = []
        request_sent = False
        response_done = asyncio.Event()

        async def receive() -> dict[str, Any]:
            nonlocal request_sent
            if not request_sent:
                request_sent = True
                return {"type": "http.request", "body": body, "more_body": False}
            await response_done.wait()
            return {"type": "http.disconnect"}

        async def send(message: dict[str, Any]) -> None:
            nonlocal status_code, response_headers
            if message["type"] == "http.response.start":
                status_code = message["status"]
                response_headers = message.get("headers", [])
            elif message["type"] == "http.response.body":
                chunk = message.get("body", b"")
                if chunk:
                    body_parts.append(chunk)
                if not message.get("more_body", False):
                    response_done.set()

        await self._app(scope, receive, send)
        assert status_code is not None
        return _AsgiResponse(status_code=status_code, body=b"".join(body_parts), headers=response_headers)

    async def get(self, path: str, *, headers: dict[str, str] | None = None) -> _AsgiResponse:
        return await self.request("GET", path, headers=headers)

    async def post(
        self,
        path: str,
        *,
        json_body: dict[str, Any] | None = None,
        headers: dict[str, str] | None = None,
    ) -> _AsgiResponse:
        return await self.request("POST", path, json_body=json_body, headers=headers)

    async def delete(self, path: str, *, headers: dict[str, str] | None = None) -> _AsgiResponse:
        return await self.request("DELETE", path, headers=headers)


# ── Middleware Tests ──────────────────────────────────────


class TestInboundRequestLoggingMiddleware:
    """Inbound request logging middleware tests."""

    @pytest.mark.asyncio
    async def test_logs_request_start_and_completion_info(self, caplog: pytest.LogCaptureFixture):
        """Middleware logs start and completion at INFO for successful requests."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post("/responses", json_body={"model": "m"})
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        start_msgs = [m for m in messages if "Inbound POST /responses started" in m]
        assert len(start_msgs) >= 1, f"Expected start log, got: {messages}"

        completion_msgs = [m for m in messages if "Inbound POST /responses completed" in m]
        assert len(completion_msgs) >= 1, f"Expected completion log, got: {messages}"
        assert "200" in completion_msgs[0]

    @pytest.mark.asyncio
    async def test_logs_warning_for_4xx_status(self, caplog: pytest.LogCaptureFixture):
        """Middleware logs WARNING for 4xx responses."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.get(f"/responses/{_NONEXISTENT_ID}")
        assert resp.status_code == 404

        warning_records = [
            r
            for r in caplog.records
            if r.name == LOGGER_NAME
            and r.levelno == logging.WARNING
            and "Inbound" in r.message
            and "completed" in r.message
        ]
        assert len(warning_records) >= 1, "Expected WARNING for 404"
        assert "404" in warning_records[0].message

    @pytest.mark.asyncio
    async def test_correlation_headers_in_log(self, caplog: pytest.LogCaptureFixture):
        """Middleware includes x-request-id and x-ms-client-request-id in log."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post(
                "/responses",
                json_body={"model": "m"},
                headers={
                    "x-request-id": "req-abc-123",
                    "x-ms-client-request-id": "client-xyz",
                },
            )
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        start_msgs = [m for m in messages if "Inbound" in m and "started" in m]
        assert len(start_msgs) >= 1
        assert "x-request-id: req-abc-123" in start_msgs[0]
        assert "x-ms-client-request-id: client-xyz" in start_msgs[0]

    @pytest.mark.asyncio
    async def test_trace_id_extracted_from_traceparent(self, caplog: pytest.LogCaptureFixture):
        """Middleware extracts trace-id from traceparent header when no OTel span is active."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        trace_id = "0af7651916cd43dd8448eb211c80319c"
        traceparent = f"00-{trace_id}-b7ad6b7169203331-01"
        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post(
                "/responses",
                json_body={"model": "m"},
                headers={"traceparent": traceparent},
            )
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        start_msgs = [m for m in messages if "Inbound" in m and "started" in m]
        assert len(start_msgs) >= 1
        assert f"trace-id: {trace_id}" in start_msgs[0]

    @pytest.mark.asyncio
    async def test_query_string_not_logged(self, caplog: pytest.LogCaptureFixture):
        """Middleware logs path only, not query string."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            await client.get(f"/responses/{_NONEXISTENT_ID}?stream=true")
        # The response should be 404 but that's fine — we're checking the logs
        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME and "Inbound" in r.message]
        assert len(messages) >= 1
        for msg in messages:
            assert "stream=true" not in msg, f"Query string leaked into log: {msg}"
            # Path should be present
            assert "/responses/" in msg

    @pytest.mark.asyncio
    async def test_duration_in_completion_log(self, caplog: pytest.LogCaptureFixture):
        """Middleware includes duration in completion log."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            await client.post("/responses", json_body={"model": "m"})

        completion_msgs = [
            r.message
            for r in caplog.records
            if r.name == LOGGER_NAME and "Inbound" in r.message and "completed" in r.message
        ]
        assert len(completion_msgs) >= 1
        assert "ms" in completion_msgs[0], f"Expected duration in ms: {completion_msgs[0]}"


# ── Handler Diagnostic Logging Tests ──────────────────────


class TestHandlerDiagnosticLogging:
    """Handler-level diagnostic logging tests."""

    @pytest.mark.asyncio
    async def test_create_logs_parameters(self, caplog: pytest.LogCaptureFixture):
        """POST /responses logs creation parameters."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post("/responses", json_body={"model": "test-model"})
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        create_msgs = [m for m in messages if "Creating response" in m]
        assert len(create_msgs) >= 1, f"Expected 'Creating response' log, got: {messages}"
        msg = create_msgs[0]
        assert "streaming=" in msg
        assert "background=" in msg
        assert "store=" in msg
        assert "model=" in msg

    @pytest.mark.asyncio
    async def test_create_sync_logs_completion(self, caplog: pytest.LogCaptureFixture):
        """Synchronous POST /responses logs response completion."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post("/responses", json_body={"model": "m"})
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        completed_msgs = [m for m in messages if "completed: status=" in m]
        assert len(completed_msgs) >= 1, f"Expected completion log, got: {messages}"
        assert "output_count=" in completed_msgs[0]

    @pytest.mark.asyncio
    async def test_get_logs_response_retrieval(self, caplog: pytest.LogCaptureFixture):
        """GET /responses/{id} logs entry and retrieval."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        # Create a response first
        create_resp = await client.post("/responses", json_body={"model": "m"})
        assert create_resp.status_code == 200
        response_id = create_resp.json()["id"]

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.get(f"/responses/{response_id}")
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        get_msgs = [m for m in messages if f"Getting response {response_id}" in m]
        assert len(get_msgs) >= 1, f"Expected GET log, got: {messages}"

        retrieved_msgs = [m for m in messages if f"Retrieved response {response_id}" in m]
        assert len(retrieved_msgs) >= 1, f"Expected retrieval log, got: {messages}"

    @pytest.mark.asyncio
    async def test_get_sse_replay_logs(self, caplog: pytest.LogCaptureFixture):
        """GET /responses/{id}?stream=true logs SSE replay entry."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            # Non-existent but valid format — just check the log message
            await client.get(f"/responses/{_NONEXISTENT_ID}?stream=true")

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        sse_msgs = [m for m in messages if "with SSE replay" in m]
        assert len(sse_msgs) >= 1, f"Expected SSE replay log, got: {messages}"

    @pytest.mark.asyncio
    async def test_delete_logs_entry_and_success(self, caplog: pytest.LogCaptureFixture):
        """DELETE /responses/{id} logs entry and success."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        create_resp = await client.post("/responses", json_body={"model": "m"})
        assert create_resp.status_code == 200
        response_id = create_resp.json()["id"]

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.delete(f"/responses/{response_id}")
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        delete_entry = [m for m in messages if f"Deleting response {response_id}" in m]
        assert len(delete_entry) >= 1, f"Expected delete entry log, got: {messages}"

        delete_success = [m for m in messages if f"Deleted response {response_id}" in m]
        assert len(delete_success) >= 1, f"Expected delete success log, got: {messages}"

    @pytest.mark.asyncio
    async def test_cancel_logs_entry(self, caplog: pytest.LogCaptureFixture):
        """POST /responses/{id}/cancel logs cancellation entry."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            # Cancel on a non-existent response — just verify the entry log fires
            await client.post(f"/responses/{_NONEXISTENT_ID}/cancel")

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        cancel_msgs = [m for m in messages if "Cancelling response" in m]
        assert len(cancel_msgs) >= 1, f"Expected cancel entry log, got: {messages}"

    @pytest.mark.asyncio
    async def test_input_items_logs_entry(self, caplog: pytest.LogCaptureFixture):
        """GET /responses/{id}/input_items logs entry."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            await client.get(f"/responses/{_NONEXISTENT_ID}/input_items")

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        input_msgs = [m for m in messages if "Getting input items" in m]
        assert len(input_msgs) >= 1, f"Expected input items log, got: {messages}"


# ── Orchestrator Handler Invocation Logging Tests ─────────


class TestOrchestratorHandlerLogging:
    """Orchestrator-level handler invocation logging."""

    @pytest.mark.asyncio
    async def test_invoking_handler_logged(self, caplog: pytest.LogCaptureFixture):
        """Orchestrator logs 'Invoking handler' with handler name."""
        app = _make_app()
        client = _AsyncAsgiClient(app)

        with caplog.at_level(logging.INFO, logger=LOGGER_NAME):
            resp = await client.post("/responses", json_body={"model": "m"})
        assert resp.status_code == 200

        messages = [r.message for r in caplog.records if r.name == LOGGER_NAME]
        handler_msgs = [m for m in messages if "Invoking handler" in m]
        assert len(handler_msgs) >= 1, f"Expected handler invocation log, got: {messages}"
        # Should include the response ID
        response_id = resp.json()["id"]
        assert response_id in handler_msgs[0]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python dedicated_host_group_update_maximum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.dedicated_host_groups.update(
        resource_group_name="rgcompute",
        host_group_name="aaaa",
        parameters={
            "properties": {
                "instanceView": {
                    "hosts": [
                        {
                            "availableCapacity": {"allocatableVMs": [{"count": 26, "vmSize": "aaaaaaaaaaaaaaaaaaaa"}]},
                            "statuses": [
                                {
                                    "code": "aaaaaaaaaaaaaaaaaaaaaaa",
                                    "displayStatus": "aaaaaa",
                                    "level": "Info",
                                    "message": "a",
                                    "time": "2021-11-30T12:58:26.522Z",
                                }
                            ],
                        }
                    ]
                },
                "platformFaultDomainCount": 3,
                "supportAutomaticPlacement": True,
            },
            "tags": {"key9921": "aaaaaaaaaa"},
            "zones": ["aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"],
        },
    )
    print(response)


# x-ms-original-file: 2025-11-01/dedicatedHostExamples/DedicatedHostGroup_Update_MaximumSet_Gen.json
if __name__ == "__main__":
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import math
import time
import asyncio

from devtools_testutils.perfstress_tests import PerfStressTest


# Used for verifying the perf framework correctly computes average throughput across parallel tests of different speed
class SleepTest(PerfStressTest):

    def __init__(self, arguments):
        super().__init__(arguments)
        self.seconds_per_operation = (self.args.initial_delay_ms / 1000) * math.pow(
            self.args.instance_growth_factor, self._parallel_index
        )

    def run_sync(self):
        time.sleep(self.seconds_per_operation)
        self.seconds_per_operation *= self.args.iteration_growth_factor

    async def run_async(self):
        await asyncio.sleep(self.seconds_per_operation)
        self.seconds_per_operation *= self.args.iteration_growth_factor

    @staticmethod
    def add_arguments(parser):
        super(SleepTest, SleepTest).add_arguments(parser)
        parser.add_argument(
            "--initial-delay-ms", nargs="?", type=int, default=1000, help="Initial delay (in milliseconds)"
        )

        # Used for verifying the perf framework correctly computes average throughput across parallel tests of different speed.
        # Each instance of this test completes operations at a different rate, to allow for testing scenarios where
        # some instances are still waiting when time expires.
        parser.add_argument(
            "--instance-growth-factor",
            nargs="?",
            type=float,
            default=1,
            help="Instance growth factor.  The delay of instance N will be (InitialDelayMS * (InstanceGrowthFactor ^ InstanceCount)).",
        )

        parser.add_argument(
            "--iteration-growth-factor",
            nargs="?",
            type=float,
            default=1,
            help="Iteration growth factor.  The delay of iteration N will be (InitialDelayMS * (IterationGrowthFactor ^ IterationCount)).",
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# ---------------------------------------------------------
"""Auto-discover tools from ``.github/tools/`` directories.

Each tool is a directory containing:

* ``TOOL.md`` -- YAML frontmatter with *name*, *description*, *parameters*
* ``tool.py`` -- Python module exporting an async ``handler(invocation)``

Discovery mirrors the skills pattern: scan ``tools/*/TOOL.md``, parse the
YAML frontmatter for the schema, and dynamically import ``tool.py`` for the
handler.
"""

import importlib.util
import logging
import pathlib
import sys
from typing import Any, List, Tuple

import yaml

from copilot.tools import Tool

logger = logging.getLogger(__name__)


def _parse_frontmatter(path: pathlib.Path) -> dict:
    """Extract YAML frontmatter from a markdown file."""
    text = path.read_text(encoding="utf-8")
    if not text.startswith("---"):
        return {}
    end = text.index("---", 3)
    return yaml.safe_load(text[3:end]) or {}


def _load_handler(tool_dir: pathlib.Path) -> Any:
    """Dynamically import ``tool.py`` and return the module."""
    tool_py = tool_dir / "tool.py"
    if not tool_py.exists():
        raise FileNotFoundError(f"No tool.py in {tool_dir}")

    module_name = f"_tool_{tool_dir.name}"
    spec = importlib.util.spec_from_file_location(module_name, str(tool_py))
    module = importlib.util.module_from_spec(spec)  # type: ignore[arg-type]
    sys.modules[module_name] = module
    spec.loader.exec_module(module)  # type: ignore[union-attr]

    if not hasattr(module, "handler"):
        raise AttributeError(f"tool.py in {tool_dir} must define an async 'handler' function")

    return module


def discover_tools(project_root: pathlib.Path) -> Tuple[List[Tool], List[Any]]:
    """Scan ``.github/tools/`` for tool directories and return SDK Tool objects.

    :param project_root: Root directory of the agent project.
    :return: ``(tools, modules)`` -- list of :class:`copilot.Tool` objects and
        their loaded Python modules.
    """
    tools_dir = project_root / ".github" / "tools"
    if not tools_dir.exists():
        return [], []

    tools: List[Tool] = []
    modules: List[Any] = []

    for tool_dir in sorted(tools_dir.iterdir()):
        tool_md = tool_dir / "TOOL.md"
        if not tool_dir.is_dir() or not tool_md.exists():
            continue

        try:
            meta = _parse_frontmatter(tool_md)
            name = meta.get("name", tool_dir.name)
            description = meta.get("description", "")
            parameters = meta.get("parameters", {})

            module = _load_handler(tool_dir)

            tool = Tool(
                name=name,
                description=description,
                parameters=parameters,
                handler=module.handler,
            )
            tools.append(tool)
            modules.append(module)
            logger.info("Discovered tool: %s (%s)", name, tool_dir)

        except Exception:
            logger.warning("Failed to load tool from %s", tool_dir, exc_info=True)

    return tools, modules
